@@ -1,6 +1,6 @@
 ########### is.pconsecutive ##############
 # little helper function to determine if the time periods of an object are consecutive per id.
-# By consecutive we mean "consecutive in the numbers", i.e. is.pconsecutive takes the numerical
+# By consecutive we mean "consecutive in the numbers", i.e., is.pconsecutive takes the numerical
 # value of the time variable into account: t, t+1, t+2, ... where t is an integer
 #
 # For this, we need as.numeric(as.character(time_var)) where as.character is a crucial part!
@@ -29,7 +29,7 @@
 #' observations for an individuals in the (p)data.frame/pseries (it is not
 #' about "neighbouring" rows). If the object to be evaluated is a pseries or a
 #' pdata.frame, the time index is coerced from factor via as.character to
-#' numeric, i.e. the series
+#' numeric, i.e., the series
 #' `as.numeric(as.character(index(<pseries/pdata.frame>)[[2]]))]` is
 #' evaluated for gaps.
 #' 
@@ -74,10 +74,10 @@
 #'     time identifiers, each as stacked time series,
 #' @param \dots further arguments.
 #' @return A named `logical` vector (names are those of the
-#'     individuals).  The i-th element of the returned vector
-#'     corresponds to the i-th individual.  The values of the i-th
-#'     element can be: \item{"TRUE"}{if the i-th individual has
-#'     consecutive time periods,} \item{"FALSE"}{if the i-th
+#'     individuals). The i-th element of the returned vector
+#'     corresponds to the i-th individual. The values of the i-th
+#'     element can be: \item{TRUE}{if the i-th individual has
+#'     consecutive time periods,} \item{FALSE}{if the i-th
 #'     individual has non-consecutive time periods,}
 #'     \item{"NA"}{if there are any NA values in time index of
 #'     the i-th the individual; see also argument `na.rm.tindex`
@@ -165,11 +165,11 @@ is.pconsecutive.default <- function(x, id, time, na.rm.tindex = FALSE, ...) {
   # argument 'x' just used for input check (if it is not NULL and is a vector)
   
   # input checks
-  if (length(id) != length(time)) 
+  if(length(id) != length(time)) 
     stop(paste0("arguments 'id' and 'time' must have same length: length(id): ", length(id), ", length(time) ", length(time)))
   
-  if (!is.null(x) && is.vector(x)) {
-    if (!(length(x) == length(id) && length(x) == length(time) && length(id) == length(time)))
+  if(!is.null(x) && is.vector(x)) { # is.vector could be too strict? factor is not a vector
+    if(!(length(x) == length(id) && length(x) == length(time) && length(id) == length(time)))
       stop(paste0("arguments 'x', 'id', 'time' must have same length: length(x): ", 
                   length(x), ", length(id): ", length(id), ", length(time): ", length(time)))
     
@@ -182,7 +182,7 @@ is.pconsecutive.default <- function(x, id, time, na.rm.tindex = FALSE, ...) {
   #   is.pconsecutive.pdata.frame or is.pconsecutive.pseries as a pdata.frame (which is sorted) was constructed 
   #   in the first place; for data.frame interface the ordering is done in the respective function
   
-  if (na.rm.tindex) {
+  if(na.rm.tindex) {
     NA_tindex <- is.na(time)
     time <- time[!NA_tindex]
     id <- id[!NA_tindex]
@@ -193,40 +193,41 @@ is.pconsecutive.default <- function(x, id, time, na.rm.tindex = FALSE, ...) {
   #  see R FAQ 7.10 for coercing factors to numeric: 
   #      as.numeric(levels(factor_var))[as.integer(factor_var)]   is more efficient than
   #      as.numeric(as.character(factor_var))
-  if (!is.numeric(time) && is.factor(time)) time <- as.numeric(levels(time))[as.integer(time)]
+  if(!is.numeric(time) && is.factor(time)) time <- as.numeric(levels(time))[as.integer(time)]
   
   list_id_timevar <- split(time, id, drop = TRUE)
 
-  res <- sapply(list_id_timevar, function(id_timevar) { if(anyNA(id_timevar)) {
+  res <- vapply(list_id_timevar, function(id_timevar) { if(anyNA(id_timevar)) {
                                                            NA # return NA if NA found in the time periods for individual
                                                           } else {
-                                                              begin <- id_timevar[1]
+                                                              begin <- id_timevar[1L]
                                                               end   <- id_timevar[length(id_timevar)]
-                                                 
+                                                              
                                                               # compare to length(original id_timevar) to find out if times are consecutive
-                                                              (end - begin + 1) == length(id_timevar)
+                                                              (end - begin + 1L) == length(id_timevar)
                                                               
                                                               # Alternative way of checking:
                                                                 # consecutive time periods from begin to end (if id_timevar were consecutive)
                                                                 # consecutive <- seq(from = begin, to = end, by = 1)
                                                                 # length(consecutive) == length(id_timevar)
                                                           }
-                                                      })
+                                                      }, FUN.VALUE = TRUE)
+  
   return(res)
 }
 
 #' @rdname is.pconsecutive
 #' @export
 is.pconsecutive.data.frame <- function(x, index = NULL, na.rm.tindex = FALSE, ...){
-  if (!is.null(index) & length(index) != 2)
+  if (!is.null(index) && length(index) != 2L)
     stop("if argument 'index' is not NULL, 'index' needs to specify
          'individual' and 'time' dimension for is.pconsecutive to work on a data.frame")
   
-  if (is.null(index)) index_orig_names <- names(x)[1:2] # assume first two columns to be the index vars
-    else index_orig_names <- index
+  # if index not provided, assume first two columns to be the index vars
+  index_orig_names <- if(is.null(index)) names(x)[1:2] else index
   
-  id   <- x[ , index_orig_names[1]]
-  time <- x[ , index_orig_names[2]]
+  id   <- x[ , index_orig_names[1L]]
+  time <- x[ , index_orig_names[2L]]
 
   # order as stacked time series (by id and time) first, otherwise default method does not work correctly!
   ord <- order(id, time)
@@ -243,29 +244,23 @@ is.pconsecutive.data.frame <- function(x, index = NULL, na.rm.tindex = FALSE, ..
 #' @rdname is.pconsecutive
 #' @export
 is.pconsecutive.pseries <- function(x, na.rm.tindex = FALSE, ...){
-  index <- attr(x, "index")
-  id   <- index[[1]]
-  time <- index[[2]]
-  return(is.pconsecutive.default(x, id, time, na.rm.tindex = na.rm.tindex, ...))
+  index <- unclass(attr(x, "index")) # unclass for speed
+  return(is.pconsecutive.default(x, index[[1L]], index[[2L]], na.rm.tindex = na.rm.tindex, ...))
 }
 
 
 #' @rdname is.pconsecutive
 #' @export
 is.pconsecutive.pdata.frame <- function(x, na.rm.tindex = FALSE, ...){
-  index <- attr(x, "index")
-  id   <- index[[1]]
-  time <- index[[2]]
-  return(is.pconsecutive.default(x, id, time, na.rm.tindex = na.rm.tindex, ...))
+  index <- unclass(attr(x, "index")) # unclass for speed
+  return(is.pconsecutive.default(x, index[[1L]], index[[2L]], na.rm.tindex = na.rm.tindex, ...))
 }
 
 #' @rdname is.pconsecutive
 #' @export
 is.pconsecutive.panelmodel <- function(x, na.rm.tindex = FALSE, ...){
-  index <- attr(x$model, "index")
-  id   <- index[[1]]
-  time <- index[[2]]
-  return(is.pconsecutive.default(x, id, time, na.rm.tindex = na.rm.tindex, ...))
+  index <- unclass(attr(x$model, "index")) # unclass for speed
+  return(is.pconsecutive.default(x, index[[1L]], index[[2L]], na.rm.tindex = na.rm.tindex, ...))
 }
 
 
@@ -280,7 +275,7 @@ is.pconsecutive.panelmodel <- function(x, na.rm.tindex = FALSE, ...){
 
 #' Check if data are balanced
 #' 
-#' This function checks if the data are balanced, i.e. if each individual has
+#' This function checks if the data are balanced, i.e., if each individual has
 #' the same time periods
 #' 
 #' Balanced data are data for which each individual has the same time periods.
@@ -292,8 +287,9 @@ is.pconsecutive.panelmodel <- function(x, na.rm.tindex = FALSE, ...){
 #' 
 #' @aliases is.pbalanced
 #' @param x an object of class `pdata.frame`, `data.frame`,
-#'     `pseries`, `panelmodel`, `pgmm`;
-#' @param y **to describe**
+#'     `pseries`, `panelmodel`, or `pgmm`,
+#' @param y (only in default method) the time index variable (2nd index
+#' variable),
 #' @param index only relevant for `data.frame` interface; if
 #'     `NULL`, the first two columns of the data.frame are
 #'     assumed to be the index variables; if not `NULL`, both
@@ -339,16 +335,12 @@ is.pbalanced <- function(x, ...) {
 #' @rdname is.pbalanced
 #' @export
 is.pbalanced.default <- function(x, y, ...) {
-  if (length(x) != length(y)) stop("The length of the two vectors differs\n")
+  if (length(x) != length(y)) stop("The length of the two inputs differs\n")
   x <- x[drop = TRUE] # drop unused factor levels so that table 
   y <- y[drop = TRUE] # gives only needed combinations
   z <- table(x, y)
-  if (any(as.vector(z) == 0)) {
-    balanced <- FALSE
-  } else { balanced <- TRUE }
-  
-  if (any(as.vector(z) > 1)) warning("duplicate couples (id-time)\n")
-  
+  balanced <- if(any(v <- as.vector(z) == 0L)) FALSE else TRUE
+  if (any(v > 1L)) warning("duplicate couples (id-time)\n")
   return(balanced)
 }
 
@@ -356,24 +348,22 @@ is.pbalanced.default <- function(x, y, ...) {
 #' @export
 is.pbalanced.data.frame <- function(x, index = NULL, ...) {
   x <- pdata.frame(x, index)
-  index <- attr(x, "index")
-  id <- index[[1]]
-  time <- index[[2]]
-  return(is.pbalanced(id, time))
+  index <- unclass(attr(x, "index")) # unclass for speed
+  return(is.pbalanced(index[[1L]], index[[2L]]))
 }
 
 #' @rdname is.pbalanced
 #' @export
 is.pbalanced.pdata.frame <- function(x, ...) {
-  index <- attr(x, "index")
-  return(is.pbalanced(index[[1]], index[[2]]))
+  index <- unclass(attr(x, "index")) # unclass for speed
+  return(is.pbalanced(index[[1L]], index[[2L]]))
 }
 
 #' @rdname is.pbalanced
 #' @export
 is.pbalanced.pseries <- function(x, ...) {
-  index <- attr(x, "index")
-  return(is.pbalanced(index[[1]], index[[2]]))
+  index <- unclass(attr(x, "index")) # unclass for speed
+  return(is.pbalanced(index[[1L]], index[[2L]]))
 }
 
 #' @rdname is.pbalanced
