@@ -1,6 +1,6 @@
 # model.frame method for pdata.frame ; the formula argument must be a
 # pdata.frame and the data argument must be a formula, which is quite
-# hesoteric, but consistent with the argument list of
+# esoteric, but consistent with the argument list of
 # model.frame.Formula which is latter called.
 
 
@@ -53,14 +53,14 @@
 #' data("Grunfeld", package = "plm")
 #' pGrunfeld <- pdata.frame(Grunfeld)
 #' 
-#' # then make a model frame from a pFormula and a pdata.frame
+#' # then make a model frame from a formula and a pdata.frame
 ##pform <- pFormula(inv ~ value + capital)
 ##mf <- model.frame(pform, data = pGrunfeld)
 #' form <- inv ~ value
 #' mf <- model.frame(pGrunfeld, form)
 #' 
 #' # then construct the (transformed) model matrix (design matrix)
-#' # from formula and model frame
+#' # from model frame
 ##modmat <- model.matrix(pform, data = mf, model = "within")
 #' modmat <- model.matrix(mf, model = "within")
 #' 
@@ -82,8 +82,8 @@ model.frame.pdata.frame <- function(formula, data = NULL, ...,
                                     lhs = NULL, rhs = NULL, dot = "previous"){
     pdata <- formula
     formula <- as.Formula(data)
-    if (is.null(rhs)) rhs <- 1:(length(formula)[2])
-    if (is.null(lhs)) lhs <- ifelse(length(formula)[1] > 0, 1, 0)
+    if (is.null(rhs)) rhs <- 1:(length(formula)[2L])
+    if (is.null(lhs)) lhs <- if(length(formula)[1L] > 0L) 1 else 0
     index <- attr(pdata, "index")
     mf <- model.frame(formula, as.data.frame(pdata, row.names = FALSE), ..., # NB need row.names = FALSE to ensure mf has integer sequence as row names
                       lhs = lhs, rhs = rhs, dot = dot)
@@ -111,9 +111,9 @@ formula.pdata.frame <- function(x, ...){
 #' @export
 model.matrix.plm <- function(object, ...){
     dots <- list(...)
-    model <- ifelse(is.null(dots$model), describe(object, "model"), dots$model)
-    effect <- ifelse(is.null(dots$effect), describe(object, "effect"), dots$effect)
-    rhs <- ifelse(is.null(dots$rhs), 1, dots$rhs)
+    model  <- if(is.null(dots$model))  describe(object, "model")  else dots$model
+    effect <- if(is.null(dots$effect)) describe(object, "effect") else dots$effect
+    rhs    <- if(is.null(dots$rhs)) 1 else dots$rhs
     cstcovar.rm <- dots$cstcovar.rm
     formula <- formula(object)
     data <- model.frame(object)
@@ -147,7 +147,7 @@ model.matrix.pdata.frame <- function(object,
     data <- object
     has.intercept <- has.intercept(formula, rhs = rhs)
     # relevant defaults for cstcovar.rm
-    if (is.null(cstcovar.rm)) cstcovar.rm <- ifelse(model == "within", "intercept", "none")
+    if(is.null(cstcovar.rm)) cstcovar.rm <- if(model == "within") "intercept" else "none"
     balanced <- is.pbalanced(data)
     X <- model.matrix(as.Formula(formula), data = data, rhs = rhs, dot = "previous", ...)
     # check for infinite or NA values and exit if there are some
@@ -156,43 +156,45 @@ model.matrix.pdata.frame <- function(object,
                    "values (NA/NaN/Inf/-Inf)"))
     X.assi <- attr(X, "assign")
     X.contr <- attr(X, "contrasts")
-    X.contr <- X.contr[ ! sapply(X.contr, is.null) ]
+    X.contr <- X.contr[ ! vapply(X.contr, is.null, FUN.VALUE = TRUE, USE.NAMES = FALSE) ]
     index <- index(data)
     attr(X, "index") <- index
-    if (effect == "twoways" && model %in% c("between", "fd"))
-        stop("twoways effect only relevant for within, random and pooling models")
-    if (model == "within") X <- Within(X, effect)
-    if (model == "Sum") X <- Sum(X, effect)
-    if (model == "Between") X <- Between(X, effect)
-    if (model == "between") X <- between(X, effect)
-    if (model == "mean") X <- Mean(X)
-    if (model == "fd") X <- pdiff(X, effect = "individual",
-                                  has.intercept = has.intercept)
-    if (model == "random"){
-        if (is.null(theta)) stop("a theta argument should be provided")
-        if (effect %in% c("time", "individual")) X <- X - theta * Between(X, effect)
-        if (effect == "nested") X <- X - theta$id * Between(X, "individual") -
+    if(effect == "twoways" && model %in% c("between", "fd"))
+        stop("twoways effect only relevant for within, random, and pooling models")
+    if(model == "within")  X <- Within(X, effect)
+    if(model == "Sum")     X <- Sum(X, effect)
+    if(model == "Between") X <- Between(X, effect)
+    if(model == "between") X <- between(X, effect)
+    if(model == "mean")    X <- Mean(X)
+    if(model == "fd")      X <- pdiff(X, effect = "individual",
+                                   has.intercept = has.intercept)
+    if(model == "random"){
+        if(is.null(theta)) stop("a theta argument must be provided for model = \"random\"")
+        if(effect %in% c("time", "individual")) X <- X - theta * Between(X, effect)
+        if(effect == "nested") X <- X - theta$id * Between(X, "individual") -
                                     theta$gp * Between(X, "group")
-        if (effect == "twoways" && balanced)
+        if(effect == "twoways" && balanced)
             X <- X - theta$id * Between(X, "individual") -
                 theta$time * Between(X, "time") + theta$total * Mean(X)
+        ## TODO: case unbalanced twoways not treated here. Catch and error gracefully?
+        # if (effect == "twoways" && !balanced) stop("two-way unbalanced case not implemented in model.matrix.pdata.frame")
     }
-    if (cstcovar.rm == "intercept"){
+    if(cstcovar.rm == "intercept"){
         posintercept <- match("(Intercept)", colnames(X))
-        if (! is.na(posintercept)) X <- X[, - posintercept, drop = FALSE]
+        if (! is.na(posintercept)) X <- X[ , - posintercept, drop = FALSE]
     }
-    if (cstcovar.rm %in% c("covariates", "all")){
+    if(cstcovar.rm %in% c("covariates", "all")){
         cols <- apply(X, 2, is.constant)
         cstcol <- names(cols)[cols]
         posintercept <- match("(Intercept)", cstcol)
-        cstintercept <- ifelse(is.na(posintercept), FALSE, TRUE)
-        zeroint <- ifelse(cstintercept &&
-                          max(X[, posintercept]) < sqrt(.Machine$double.eps),
-                          TRUE, FALSE)
-        if (length(cstcol) > 0){
-            if ((cstcovar.rm == "covariates" || !zeroint) && cstintercept) cstcol <- cstcol[- posintercept]
-            if (length(cstcol) > 0){
-                X <- X[, - match(cstcol, colnames(X)), drop = FALSE]
+        cstintercept <- if(is.na(posintercept)) FALSE else TRUE
+        zeroint <- if(cstintercept &&
+                          max(X[ , posintercept]) < sqrt(.Machine$double.eps))
+                          TRUE else FALSE
+        if(length(cstcol) > 0L){
+            if((cstcovar.rm == "covariates" || !zeroint) && cstintercept) cstcol <- cstcol[- posintercept]
+            if(length(cstcol) > 0L){
+                X <- X[ , - match(cstcol, colnames(X)), drop = FALSE]
                 attr(X, "constant") <- cstcol
             }
         }
@@ -263,14 +265,12 @@ pmodel.response <- function(object, ...) {
 pmodel.response.plm <- function(object, ...){
     y <- model.response(model.frame(object))
     dots <- list(...)
-    if (is.null(dots$model)) model <- describe(object, "model") else model <- dots$model
-    if (is.null(dots$effect))
-        effect <- describe(object, "effect") else effect <- dots$effect
-    if (is.null(dots$theta)){
-        if (describe(object, "model") == "random")
-            theta <- ercomp(object)$theta else theta <- NULL
-    }
-    else theta <- dots$theta
+    model  <- if(is.null(dots$model))   describe(object, "model")  else dots$model 
+    effect <- if(is.null(dots$effect))  describe(object, "effect") else dots$effect
+    theta  <- if(is.null(dots$theta)) {
+                    if(describe(object, "model") == "random") 
+                        ercomp(object)$theta else NULL
+                } else dots$theta
     ptransform(y, model = model, effect = effect, theta = theta)
 }
 
@@ -278,11 +278,10 @@ pmodel.response.plm <- function(object, ...){
 #' @export
 pmodel.response.data.frame <- function(object, ...){
     dots <- list(...)
-    if (is.null(attr(object, "terms"))) stop("not a model.frame")
-    model <- ifelse(is.null(dots$model), "pooling", dots$model)
-    effect <- ifelse(is.null(dots$effect), "individual", dots$effect)
-    if (is.null(dots$theta)) theta <- NULL else theta <- dots$theta
-#    theta <- ifelse(is.null(dots$theta), NULL, dots$theta)
+    if(is.null(attr(object, "terms"))) stop("not a model.frame")
+    model  <- if(is.null(dots$model))  "pooling"    else dots$model
+    effect <- if(is.null(dots$effect)) "individual" else dots$effect
+    theta  <- if(is.null(dots$theta))  NULL         else dots$theta
     y <- model.response(object)
     ptransform(y, model = model, effect = effect, theta = theta)
 }
@@ -292,46 +291,47 @@ pmodel.response.data.frame <- function(object, ...){
 #' @export
 pmodel.response.formula <- function(object, data, ...){
     dots <- list(...)
-    if (is.null(data)) stop("the data argument is mandatory")
-    if (! inherits(data, "pdata.frame")) stop("the data argument must be a pdata.frame")
-    if (is.null(attr(data, "terms"))) data <- model.frame(data, object)
-    model <- dots$model
+    if(is.null(data)) stop("the data argument is mandatory")
+    if(! inherits(data, "pdata.frame")) stop("the data argument must be a pdata.frame")
+    if(is.null(attr(data, "terms"))) data <- model.frame(data, object)
+    model  <- dots$model
     effect <- dots$effect
-    theta <- dots$theta
-    if (is.null(model)) model <- "pooling"
-    if (is.null(effect)) effect <- "individual"
-    if (model == "random" && is.null(theta)) stop("the theta argument is mandatory")
+    theta  <- dots$theta
+    if(is.null(model)) model <- "pooling"
+    if(is.null(effect)) effect <- "individual"
+    if(model == "random" && is.null(theta)) stop("the theta argument is mandatory for model = \"random\"")
     y <- model.response(data)
     ptransform(y, model = model, effect = effect, theta = theta)
 }
 
 ptransform <- function(x, model = NULL, effect = NULL, theta = NULL, ...){
-    if (model == "pooling") return(x)
-    if (effect == "twoways" && model %in% c("between", "fd"))
-        stop("twoways effect only relevant for within, random and pooling models")
-    balanced <- is.pbalanced(x) # need to check this right here as long as x is a pseries
-    if (model == "within") x <- Within(x, effect)
-    if (model == "between") x <- between(x, effect)
-    if (model == "Between") x <- Between(x, effect)
-    if (model == "fd") x <- pdiff(x, "individual")
-    if (model == "random"){
-        if (is.null(theta)) stop("a theta argument should be provided")
-        if (effect %in% c("time", "individual")) x <- x - theta * Between(x, effect)
-        if (effect == "nested") x <- x - theta$id * Between(x, "individual") -
-                                    theta$gp * Between(x, "group")
-        if (effect == "twoways" && balanced)
-            x <- x - theta$id * Between(x, "individual") -
-                theta$time * Between(x, "time") + theta$total * mean(x)
+    # NB: ptransform (and hence pmodel.response) does not handle the random 2-way unbalanced case
+    
+    if(model == "pooling") return(x) # early exit
+    if(effect == "twoways" && model %in% c("between", "fd"))
+        stop("twoways effect only relevant for within, random, and pooling models")
+
+    if(model == "within")  x <- Within(x, effect)
+    if(model == "between") x <- between(x, effect)
+    if(model == "Between") x <- Between(x, effect)
+    if(model == "fd")      x <- pdiff(x, "individual")
+    if(model == "random") {
+        balanced <- is.pbalanced(x) # need to check this right here as long as x is a pseries
+        if(is.null(theta)) stop("a theta argument must be provided")
+        if(effect %in% c("time", "individual")) x <- x - theta * Between(x, effect)
+        if(effect == "nested") x <- x - theta$id * Between(x, "individual") -
+                                         theta$gp * Between(x, "group")
+        if(effect == "twoways" && balanced)
+            x <- x - theta$id   * Between(x, "individual") -
+                     theta$time * Between(x, "time") + theta$total * mean(x)
+        ## TODO: could catch non-treated case to error gracefully:
+        # if (effect == "twoways" && !balanced) warning("two-way unbalanced case not implemented in ptransform")
     }
     
-    res <- if (model %in% c("between", "fd")) {
-    # these models "compress" the data, thus an index does not make sense here -> no pseries
-               x
-           } else {
-               structure(x, index = index(x), class = union("pseries", class(x)))
-           }
-    return(res)
+    # between and fd models "compress" the data, thus an index does not make
+    # sense for those, but add to all others (incl. Between (capital B))
+    x <- if(model %in% c("between", "fd")) x
+         else structure(x, index = index(x), class = union("pseries", class(x)))
+    return(x)
 }
 
-####
-
